%
%   This function transforms the interpolation data for the
%    multiple Nevanlinna-Pick interpolation problem subjected
%    to the bilinear transformation
%
%              au + b 
%       z --> -------- 
%              cu + d  
%
%   Input:  a,b,c,d  -  Transformation parameters
%           z        -  Interpolation point
%           w        -  Interpolation data 
%           where  w = [ w0 w1 ... w(m-1) ]
%
%   Output: u  -  Interpolation point
%           v  -  Interpolation data 
%           where  v = [ v0 v1 ... v(m-1) ]
%
%   so that
%              dz - b
%       u = - --------
%              cz - a
%
%             f(z) = w0 
%            f'(z) = w1 
%                  .      
%                  .       
%                  .       
%       f(m-1)(z)
%      ----------- = w(m-1)
%        (m-1)!
%
%
%             g(u) = v0 
%            g'(u) = v1 
%                  .      
%                  .       
%                  .       
%       g(m-1)(u)
%      ----------- = v(m-1)
%        (m-1)!
%
function [u,v] = bilineardomaindatalow(a,b,c,d,z,w)

% Determine interpolation point and function value
if abs(z) > 1e10
    u = - d / c;
else
    u = - (d*z - b) / (c*z - a);
end
v(1) = w(1);

% Determine multiplicity
m = size(w,2);

if(m >= 2)
    % Determine first derivative interpolation data
    zprim = (a*d - b*c) / (c*u - a)^2;
    v(2) = w(2) * zprim;
    
    % Determine higher order derivative interpolation data (if any...)
    coeff = 1;
    fact = [];
    for k = 3:m
        coeff = conv(coeff,[1 1]);
        fact = [ factorial(k-2) fact ];
        zprim = [ zprim(end)*(k-1)/(c*u - a)*c zprim ];
        v(k) = coeff(1:end).*w(2:k).*[fact 1]*zprim';
    end
end