%
%   This function transforms the interpolation data for the
%    multiple Nevanlinna-Pick interpolation problem subjected
%    to the bilinear transformation
%
%               af(z) + b                      b - dg(z)
%       g(z) = -----------     <-->    f(z) = ----------- 
%               cf(z) + d                      cg(z) - a
%
%   Input:  a,b,c,d  -  Transformation parameters
%           w        -  Interpolation data for f 
%           where  w = [ w0 w1 ... wm-1 ]
%
%   Output: v  -  Interpolation data for g
%           where  v = [ v0 v1 ... vm-1 ]
%
%             f(z) = w0 
%            f'(z) = w1 
%                  .      
%                  .       
%                  .       
%       f(m-1)(z)
%      ----------- = w(m-1)
%        (m-1)!
%
%
%             g(z) = v0 
%            g'(z) = v1 
%                  .      
%                  .       
%                  .       
%       g(m-1)(z)
%      ----------- = v(m-1)
%        (m-1)!
%
function [v] = bilinearrangedatalow(a,b,c,d,w)

% Determine interpolation point and function value
v(1) = (a*w(1) + b) / (c*w(1) + d);

% Determine multiplicity
m = size(w,2);

if(m >= 2)
    % Determine first derivative interpolation data
    denom = c * w(1) + d;
    v(2) = w(2) * (a - c*v(1)) / denom;
    
    % Determine higher order derivative interpolation data (if any...)
    coeff = [1 1];
    fact = 1;
    for k = 3:m
        coeff = conv(coeff,[1 1]);
        fact = [ factorial(k-1) fact ];
        v(k) = ([ a zeros(size(v)-1) ] - c*v).*fliplr(w(2:k)) ...
            .*fact*coeff(1:end-1)';
    end
end