%
%   This program designs a controller for the flexible beam problem 
%    of [1] using the npdcsolver. The performance is compared with 
%    the design of [1].
%
%       [1] Doyle, Francis & Tannenbaum, "Feedback Control Theory",
%            Mcmillan, 1992.
%
clear all
format compact

% To add if multiplicity of zeros / poles

denP = [5 3.5682 139.5021 .0929 0];
numP = [-6.4750 4.0302 175.77 ];
rhppoles = roots(denP);
rhppoles = rhppoles(find(real(rhppoles)>=0 & imag(rhppoles)>=0));
rhpzeros = roots(numP);
rhpzeros = rhpzeros(find(real(rhpzeros)>=0 & imag(rhpzeros)>=0));
noofinf = length(denP) - length(numP);

z = [];
if noofinf > 0
    z = Inf;        % To add if no inf - move one to inf / origin
    w{1} = [ 1 zeros(1,noofinf)];
end

z = [ z rhppoles rhpzeros ];

for k = 1:length(rhppoles)
    w{end+1}  = 0;
end

for k = 1:length(rhpzeros)
    w{end+1}  = 1;
end

%
%   Transforming the standard form, i.e. from unit disc to 
%    right half plane.
%

% best 2002/1/30
gamma = 1.8;
kappa = 0.9;
speczeros = [ 1.7*i 7 1e2 ];

a = -kappa; b = kappa; c = 1; d = 1;
domaintrans = getbltrans(a,b,c,d);
rangetrans = getbltrans(1,gamma,-1,gamma);

interdata1 = getinterdata(z,w,speczeros);
interdata2 = bilineardomaindata(domaintrans,interdata1);
interdata3 = bilinearrangedata(rangetrans,interdata2);

interpolant3 = npdc(interdata3);
interror(interpolant3.a,interpolant3.b,interdata3)

interpolant2 = bilinearrange(rangetrans,interpolant3);
interror(interpolant2.a,interpolant2.b,interdata2)

interpolant1 = bilineardomain(domaintrans,interpolant2);
interror(interpolant1.a,interpolant1.b,interdata1)


%
%   Compare with the DFT solution
%

numQim = [0.0008 0.0221 0.1768 0.7007 3.8910 0.0026 0];
denQim = [1 6.1081 6.8897 4.9081];
numJ = 1;
tau = 0.04;
tau1 = [tau 1];
denJ = conv(conv(tau1,tau1),tau1);
numQ = conv(numQim,numJ);
denQ = conv(denQim,denJ);
sysQ = tf(numQ,denQ);
sysP = tf(numP,denP);
sysC = feedback(sysQ,sysP,+1);
sysC = minreal(sysC);

sysPC = series(sysC,sysP);
sysS = feedback(1,sysPC);
wS = 10.^(-4:.04:4)';
[svS,wS] = sigma(sysS,wS);

numC2 = [1.424 9.076e2 3.141e4 1.117e5 9.073e5 1.961e6 1.306e3 .01406 ];
denC2 = [1 1.013e3 1.326e4 1.129e5 6.326e5 2.348e6 4.940e6 3.440e6 3.435e3 ];
sysC2 = tf(numC2,denC2);
sysPC2 = series(sysC2,sysP);
sysS2 = feedback(1,sysPC2);
[svS2,wS] = sigma(sysS2,wS);

sysSnpdc = tf(fliplr(interpolant1.b),fliplr(interpolant1.a));
sysCnpdc = series(inv(series(sysP,sysSnpdc)),parallel(1,-sysSnpdc));
sysCnpdc = minreal(sysCnpdc,1e-4);

% Special case
[numCnpdc,denCnpdc] = tfdata(sysCnpdc,'v')
while numCnpdc(1) < 1e-4
    numCnpdc = numCnpdc(2:end);
end
while denCnpdc(1) < 1e-4
    denCnpdc = denCnpdc(2:end);
end
sysCnpdc = tf(numCnpdc,denCnpdc)

sysPCnpdc = series(sysCnpdc,sysP);
sysS2npdc = feedback(1,sysPCnpdc);
[svS2npdc,wS] = sigma(sysS2npdc,wS);
[svSnpdc,wS] = sigma(sysSnpdc,wS);

numSid = [1 1.2 0];
denSid = [1 1.2 1];
sysSid = tf(numSid,denSid);
[svSid,wS] = sigma(sysSid,wS);
sysCid = series(inv(series(sysP,sysSid)),parallel(1,-sysSid));
sysCid = minreal(sysCid);


%
%   For printing
%
papersize = [ 10 7.5 ];
width = 7;
height = 5;
left = (papersize(1)- width) / 2;
bottom = (papersize(2)- height) / 2;
myfiguresize = [left, bottom, width, height];
myfontsize = 16;
myfontweight = 'bold';

figure(1), clf
set(gcf,'Units','centimeters')
set(gcf, 'PaperPosition', myfiguresize);
loglog(wS,svS2,'k--',wS,svSid,'k:',wS,svS2npdc,'k-')
legend('DFT','Ideal','NPDC',4)
title('Sensitivity Function')
changetextstyle(gcf,myfontsize,myfontweight);
print sensitivity -deps


figure(2), clf
set(gcf,'Units','centimeters')
set(gcf, 'PaperPosition', myfiguresize);
sysTid = parallel(1,-sysSid);
sysT = parallel(1,-sysS);
sysT2 = parallel(1,-sysS2);
sysTnpdc = parallel(1,-sysS2npdc);
step(sysT2,'k--',sysTnpdc,'k-',0:.01:10)
legend('DFT','NPDC',4)
changetextstyle(gcf,myfontsize,myfontweight);
print step -deps

figure(3), clf
set(gcf,'Units','centimeters')
set(gcf, 'PaperPosition', myfiguresize);
sysU = feedback(sysC,sysP);
sysU2 = feedback(sysC2,sysP);
sysUnpdc = feedback(sysCnpdc,sysP);
step(sysU2,'k--',sysUnpdc,'k-',0:.01:10)
legend('DFT','NPDC',4)
axis([0 10 -.61 .61])
changetext(gcf,'Step Response','Control Signal, Step Reference');
changetextstyle(gcf,myfontsize,myfontweight);
print stepu -deps


%
%       We get some response data
%
peaknpdc = max(svSnpdc);
peakDFT = max(svS2);
peakid = max(svSid);
wS2 = wS;
while svS2(end)>=1/sqrt(2)
    svS2(end) = [];
    bandwidthDFT = wS2(end);
    wS2(end) = [];
end
wS2 = wS;
while svSnpdc(end)>=1/sqrt(2)
    svSnpdc(end) = [];
    bandwidthnpdc = wS2(end);
    wS2(end) = [];
end
wS2 = wS;
while svSid(end)>=1/sqrt(2)
    svSid(end) = [];
    bandwidthid = wS2(end);
    wS2(end) = [];
end
[ yUDFT, tUDFT ] = step(sysU2,'k--',0:.01:10);
[ yUnpdc, tUnpdc ] = step(sysUnpdc,'k--',0:.01:10);
maxUDFT = max(abs(yUDFT));
maxUnpdc = max(abs(yUnpdc));
[ yDFT, tDFT ] = step(sysT2,'k--',0:.01:10);
[ ynpdc, tnpdc ] = step(sysTnpdc,'k--',0:.01:10);
riseDFT = min(tDFT(find(yDFT>=.9))) - min(tDFT(find(yDFT>=.1)));
risenpdc = min(tnpdc(find(ynpdc>=.9))) - min(tnpdc(find(ynpdc>=.1)));
overshootDFT = max(yDFT);
overshootnpdc = max(ynpdc);
            
while yDFT(end)>=.95 & yDFT(end) <= 1.05
    yDFT = yDFT(1:end-1);
    settlingtimeDFT = tDFT(end);
    tDFT = tDFT(1:end-1);
end
while ynpdc(end)>=.95 & ynpdc(end) <= 1.05
    ynpdc = ynpdc(1:end-1);
    settlingtimenpdc = tnpdc(end);
    tnpdc = tnpdc(1:end-1);
end


disp('  Peak Hinf  Bandwidth  Rise time  Settling  Overshoot Max u ')
disp('DFT')
disp([ peakDFT bandwidthDFT riseDFT settlingtimeDFT overshootDFT maxUDFT ])
disp('npdc')
disp([ peaknpdc bandwidthnpdc risenpdc settlingtimenpdc overshootnpdc maxUnpdc ])
disp('Ideal')
disp([ peakid bandwidthid ])
