%
%   This function returns an interpolation data set of the form
%    used in the npdcsolver.
%
%   Input: z - interpolation points
%          w - interpolation data
%          speczeros -  spectral zeros
%
%   Output: interdata - an instance of the interpolation data set
%
%    interdata.z      = [ z0 z1 ... zn ]
%    interdata.w      = { w0 w1 ... wn }
%                        wi = [ wi0 wi1 ... wi(m-1) ]
%    interdata.n      = number of interpolation points minus one
%    interdata.ntilde = number of interpolation points
%    interdata.mw     = multiplicities of each interpolation point
%    interdata.m      = number of interpolation points taken
%                        with multiplicity
%    interdata.tau    = polynomial increasing order
%                                 __      __
%                       tau(z) =  || (1 - zk z)^mk
%                               k=0..m
%    interdata.speczeros = spectral zeros
%
function interdata = getinterdata(z,w,speczeros)

% Check that the sizes are ok
if size(z,1) ~= 1 | size(w,1) ~= 1
    disp('In getinterdata: z & w must be 1 times something')
    return
elseif size(z,2) ~= size(w,2)
    disp('In getinterdata: z & w must have the same length')
    return
end

% Check that the z:s are distinct
for k = 1:size(z,2)
    for l = k+1:size(z,2)
        if abs(z(k) - z(l)) < 10*eps
            disp('In getinterdata: the z:s must be distinct')
            return
        end
    end
end

% Check that the wk:s are nonempty
for k = 1: size(w,2)
    if isempty(w{k})
        disp('In getinterdata: the wk:s must be non-empty')
        return
    end
    mw(k) = size(w{k},2);
end

% Reorder if there is a z = 0
index = find(z==0);
if index
    indexcomp = find(z~=0);
    z = z([index indexcomp]);
    w = w([index indexcomp]);
    mw = mw([index indexcomp]);
end

%   Construct tau
p = [];
noatend = 0;
for k = 1:length(z)
    if norm(z(k)) < 10*eps
        noatend = noatend + mw(k) - 1;
    elseif norm(z(k)) > 1e10
        noatend = noatend + mw(k);
    else
        p = [ p 1/conj(z(k))*ones(1,mw(k)) ];
    end
end
tau = poly(p);  % Note poly([]) = 1;
tau = real( fliplr(tau) / tau(end) );
tau = [ tau zeros(1,noatend) ];


interdata.z = z;
interdata.w = w;
interdata.ntilde = size(z,2);
interdata.n = interdata.ntilde - 1;
interdata.m = sum(mw);
interdata.tau = tau;
interdata.mw = mw;
interdata.speczeros = speczeros;