%
%   Function that calculates the hessian of the functional g_rho at
%    point alpha, muvec.
%
%   Input: alpha - [ alpha0 alpha1 ... alphan ]
%          muvec - [ mu0 mu1 ... mun ]'
%          tau   - [ tau0 tau1 ... taun ]
%          K     - modified pick matrix
%
%       where alpha(z) = alpha0 + alpha1 z + ... + alphan z^n
%             mu(z) = mu0 + mu1 (z + z^-1) + ... + mun (z^n + z^-n)
%             tau(z) = tau0 + tau1 z + ... + taun z^n
%
%   Output: h - hessian
%
%       where h(z) = g''(alpha) = 2*( K - S )
%
%
%   u = [ u0 u1 ... u2n ]'
%
%   s = [ s0 s1 ... s2n ]'
%
%   shat = [ shat-n shat-n+1 ... shat3n ]'
%
%  [ shat-n   ]   [ mun                       ] [ u0  ]
%  [ shat-n+1 ]   [ mun-1 mun                 ] [ u1  ]
%  [    .     ]   [  .        .               ] [ .   ]
%  [    .     ]   [ mu0          mun          ] [ .   ]
%  [    .     ] = [  .     .          .       ] [ .   ]
%  [    .     ]   [ mun          mu0      mun ] [     ]
%  [    .     ]   [        .          .    .  ] [     ]
%  [    .     ]   [              mun      mu0 ] [     ]
%  [    .     ]   [                   .    .  ] [     ]
%  [ shat3n   ]   [                       mun ] [ u2n ]
%
%
%  [ s0  ]   [ tn    tn-1  ...  t0    t1   ...  t3n   ] [ shat-n   ]
%  [ s1  ]   [ tn+1  tn    ...  t1    t0   ...  t3n-1 ] [ shat-n+1 ]
%  [  .  ] = [  .       .                 .      .    ] [   .      ]
%  [  .  ]   [  .          .                 .   .    ] [   .      ]
%  [ s2n ]   [ t3n   t3n-1 ...             ...  tn    ] [ shat3n   ]
%
function h = hessian3(alpha,muvec,tau,K)

m = length(alpha) - 1;

u = conv(alpha,alpha); % stable increasing
t = covar2(1,conv(u,tau),3*m);

Amu = toeplitz([fliplr(muvec') muvec(2:end)' zeros(1,2*m)],[ muvec(end) zeros(1,2*m) ]);
At = toeplitz(t((m+1):end),[fliplr(t(2:m+1)) t]);

s = At * Amu * u';
h = 2 * ( K + hankel(s(1:m+1),s(m+1:2*m+1)) );
